----------------------------------------------------------------------------
-- ADC081S021.vhd (SPI-Slave)
----------------------------------------------------------------------------
library IEEE; use IEEE.STD_LOGIC_1164.ALL;

entity ADC081S021 is
 generic( 
  tSCL: delay_length := 500 ns;    -- Taktperiode
  D_init: std_logic_vector(7 downto 0) := x"3A");

 port( 
  cs, scl:  in std_logic;
  sda    : out std_logic);
end entity;

architecture s of ADC081S021 is
 constant tAQC:  time := 350 ns; -- Minimum time required for acquisition
 constant tQUIET:time :=  50 ns; -- Quiet time
 constant tCS:   time :=  10 ns; -- Minimum CS pulse width
 constant tSU:   time :=  10 ns; -- CS setup time prior to SCL falling edge
 constant tEN:   time :=  20 ns; --Delay from CS until SDA TRI-STATE disabled
 constant tCH:   time := 0.4*tSCL; -- SCL low pulse width
 constant tCL:   time := 0.4*tSCL; --SCL high pulse width
 constant tACC:  time :=  40 ns; --Data access time after SCL falling edge
 constant tH:    time :=   7 ns; -- SCL to data valid hold time
 constant tDIS:  time :=  25 ns; -- SCL falling edge to SDA high impedance
 signal sidx: natural range 1 to 16;
 signal D: std_logic_vector(7 downto 0):=D_init;
begin

 process
  variable tQUIET0, tAQC0, tCS0, tCS1, tSCL0, tSCL1: time := 0 ns; 
 begin
  wait for 350 ns;           -- Wartezeit bis Sensor bereit

  loop

   wait until cs='0'; tCS0 := now;
   assert now-tCS1 > tCS report "tCS=" & integer'image((now-tCS1)/1 ns) 
          & "ns: Minimum CS pulse width violation "; 
   assert (now-tAQC0)   > tAQC    report "tAQC=" & integer'image((now-tAQC0)/1 ns) 
          & "ns: Minimum time required for acquisition violation"; 
   assert (now-tQUIET0) > tQUIET  report "tQuiet=" 
          & integer'image((now-tQUIET0)/1 ns) & "ns: Quiet time violation"; 
 
   sda <= '0' after tEN; 
 
   for idx in 1 to 16 loop 
    wait until scl = '0'; tSCL0:= now;
    if idx=0 then 
     assert now-tCS1 > tSU report "tSU=" & integer'image((now-tCS1)/1 ns) 
          & ": CS setup time prior to SCL falling edge violation"; 
    else
     assert now - tSCL1 > tCH report "tCH=" & integer'image((now - tSCL1)/1 ns) 
          & "ns: SCL high pulse width violation";
    end if;

    if idx>3 and idx<12 then sda <= D(11-idx) after tACC;
    elsif idx=12 then sda <= '0' after tACC;
    elsif idx=16 then 
     sda <= 'Z' after tDIS;
     tQUIET0 := now + tDIS;
    end if;
    if idx=14 then tAQC0:= now; end if;
    wait until scl = '1'; tSCL1:= now;
    assert now - tSCL0 > tCL report "tCL=" & integer'image((now - tSCL0)/1 ns) 
          & "ns: SCL low pulse width violation";
	sidx <= idx;
   end loop;

   if cs='0' then wait until cs='1';  tCS1:=now;
   else tCS1 := cs'last_event; end if;
   D <= D(6 downto 0) & (D(7) xor D(5) xor D(4) xor D(3));

  end loop;
 end process;
end architecture;

