-----------------------------------------------------------------------------
-- ps2t_dev.vhd
-----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity ps2t_dev is
 generic (tbit: delay_length := 40 us);
 port(run_sim: boolean;                 -- Simulationssteuersignal
 send: in std_logic;                    -- Sendesignal
 x: in  std_logic_vector(7 downto 0);   -- Sendedaten
 y: out std_logic_vector(7 downto 0);   -- Empfangsdaten
 ps2d, ps2t: inout std_logic:= 'Z');	-- PS/2-Signale
end entity;

architecture sim of ps2t_dev is
 signal P: std_logic;
 signal idx: natural range 0 to 15;
 type t_state is (
    BAT,  -- Startzustand (Basic Assurnce Test)
	RDY,  -- PC bereit zum Empfang, Tastatur darf senden
    BSY,  -- PC beschäftigt, Tastatur wartet auf Startbit	
	S0,   -- Startbit erkannt
    REC,  -- Empfangsmodus
    SND); -- Sendemodus
	
  signal state: t_state:=BAT;               -- Zustand

begin
 process(ps2t, ps2d, send)
  variable ps2tv, ps2dv:   std_logic:='1';
  variable tt, tCLK, tDAT: delay_length;
 begin
  if  run_sim then
  -- Kontrolle der Zeitbedingungen
  if falling_edge(ps2t) then
   tt := now - tDAT; assert tt > 5 us      -- Kontrolle der Datenvorhaltezeit
    report "tSU="&integer'image(tt/1 us)&" us: Vorhaltetzeit verletzt";
   tCLK := now;
  end if;
  if ps2d'event then                       -- Kontrolle der Datenhaltezeit
    tt := now - tCLK; tDAT := now; assert tt > 5 us 
    report "tHLD="&integer'image(tt/1 us)&" us: Haltezeit verletzt";
  end if;
  
  ps2dv := to_X01(ps2d);                   -- Abbildung von 'H' auf '1'
  ps2tv := to_X01(ps2t);
  
  case state is
   when RDY =>                             -- wenn PC bereit zu Empfang
    if ps2t'event and ps2tv = '0' then     -- bei fallender Taktflanke
	 state <= BSY;                         -- Uebergang in "Warte auf Startflanke"
	elsif send='1' then                    -- wenn Sendesignal aktiv
	 state <= SND; P <= '0'; idx <= 0;     -- Uebergang in den Sendezustand
	  ps2dv := '0';                        -- Daten nach 1/4 (siehe unten Signalzuweisung) 
	  ps2t <= '0' after tBit/2;            -- und Takt nach 1/2 Bitzeit auf null
	end if;
   when BSY =>                             -- PC beschäftigt, Tastatur warte auf Datenflanke
    if  ps2d'event and ps2d = '0' then     -- wenn fallende Datenflanke
	 state <= S0;                          -- Zustand "Startflanke erkannt"
	elsif ps2t'event and ps2tv = '1' then  -- wenn steigende Taktflanke
	 state <= RDY;                         -- Zustand "PC bereit zum Empfang"
	end if;
   when S0 =>                              -- Wenn Zustand "Startflanke erkannt"
    if ps2t'event and ps2tv = '1' then     -- und Takt nach eins wechselt
     state <= REC; idx <= 0; P <= '0';     -- Uebergang Empfangszustand
     ps2t <= '0' after tBit/2;             -- nach 1/2 Bitzeit fallende Taktflanke
    end if; 
   when REC =>                             -- wenn Empfangszustand
     if ps2t'event and ps2tv = '0' then    -- wenn fallende Taktflanke
      if idx = 0 then                      -- idx=0: Startbitkontrolle
	   assert ps2dv ='0'          report "Unzul. Startbit";
      elsif idx < 9	then                   -- idx=1...8: DatenbitUebernahmen
	   y(idx-1) <= ps2dv; P <= P xor ps2dv;
	  elsif idx = 9	then                   -- idx=9: Paritätsbitkontrolle
	   assert (ps2dv xor P) = '1' report "Paritaetsfehler";
	  elsif idx =10	then                   -- idx=10: Stoppbitkontrolle
	   assert ps2dv = '1'         report "kein Stoppbit"; 
	   ps2dv := '0';                       -- Empfangsbestaetigung setzen
	  end if;
	  ps2t <= 'Z' after tBit/2;            -- nach 1/2 Bitzeit steigende Taktflanke  
	 elsif ps2t'event and ps2tv = '1' then -- wenn steigende Taktflanke
	  if idx<11 then idx <= idx + 1;       -- idx<11: Zähler erhöhen und nach
	   ps2t <= '0' after tBit/2;           -- 1/2 Bitzeit steigende Taktflanke
	  else state <= RDY;  ps2dv := '1';    -- sonst Empfang beendet
	  end if;
	 end if;
	when SND =>                            -- wenn Sendezustand
	 if ps2t'event and ps2tv = '1' then    -- wenn steigende Taktflanke 
      if idx < 8 then                      -- idx 0 bis 7: Daten versenden
	   ps2dv := x(idx); P <= P xor ps2dv;
	  elsif idx = 8 then ps2dv := not P;   -- idx=8: Paritätsbit versenden
	  else ps2dv := '1';                   -- idx=9: Stoppbit versenden
	  end if;
	  if idx < 10 then                     -- idx<10: zählen, fallende und
	   idx <= idx +1; ps2t <= '0' after tBit/2; -- Taktflanke erzeugen
	  else state <= RDY;                   -- sonst Senden beendet
	  end if;
	 elsif ps2t'event and ps2tv = '0' then -- wenn fallend Taktflanke
	  ps2t <= 'Z' after tBit/2;            -- steigende Taktflanke erzeugen
	 end if;
	when others =>                         -- Anfangs und Einschaltzustand
	 ps2t <= 'Z'; ps2dv := '1';            -- PS2-Bus freigeben
	 if now > 1 us and ps2dv = '1' then 
	  state <= BSY;                        -- wenn PC PS2-Datenfreigegeben
	 end if;                               -- in den Zustand "Empfangsbereit wechseln
   end case;
	 
   -- Zuweisungen an ps2d um 1/4 tBIT verzögert und 'Z' statt '1'
    if ps2dv = '0' and ps2d /= '0' then ps2d <= '0' after tBit/4;
    elsif ps2dv /= '0' and ps2d = '0' then ps2d <= 'Z' after tBit/4;
    end if;
   
   end if;
  end process;
 end architecture;

   
  
 
  